@@ -3,11 +3,11 @@ Htmldate extracts original and updated publication dates from URLs and web pages
 """
 
 # meta
-__title__ = 'htmldate'
-__author__ = 'Adrien Barbaresi'
-__license__ = 'GNU GPL v3'
-__copyright__ = 'Copyright 2017-2022, Adrien Barbaresi'
-__version__ = '1.2.1'
+__title__ = "htmldate"
+__author__ = "Adrien Barbaresi"
+__license__ = "GNU GPL v3"
+__copyright__ = "Copyright 2017-2022, Adrien Barbaresi"
+__version__ = "1.3.1"
 
 
 import logging
